module Phylo.TypesExample where

import Prelude

import Affjax as AX
import Affjax.ResponseFormat as ResponseFormat
import DOM.Simple.Console (log, log2)
import Data.Array (filter)
import Data.Either (Either(..))
import Data.Eq.Generic (genericEq)
import Data.Generic.Rep (class Generic)
import Data.HTTP.Method (Method(..))
import Data.Maybe (Maybe(..), fromMaybe, maybe)
import Effect (Effect)
import Effect.Aff (Aff, launchAff, launchAff_)
import Effect.Class (liftEffect)
import Effect.Class.Console (logShow)
import Simple.JSON as JSON

type PhyloJSON = 
  { name              :: String
  , phyloDocs         :: String
  , phyloFoundations  :: String
  , phyloPeriods      :: String
  , phyloTerms        :: String
  , phyloGroups       :: String
  , phyloBranches     :: String
  , objects           :: Array PhyloObject
  , edges             :: Array PhyloEdge
  }

--type PhyloJSON = 
--  { name              :: String
--  , phyloDocs         :: String
--  , phyloFoundations  :: String
--  , phyloPeriods      :: String
--  , phyloTerms        :: String
--  , phyloGroups       :: String
--  , phyloBranches     :: String
--  , objects           :: { branches :: Array Branch
--                           periods  :: Array Period
--                           groups   :: Array Group
--                         }
--  , edges             :: { link :: Array Link
--                           ancestorLink  :: Array AncestorLink
--                           branchLink   :: Array BranchLink
--                         }
--  }
  
-- type PhyloData =
--   { name              :: String
--   , phyloDocs         :: String
--   , phyloFoundations  :: String
--   , phyloPeriods      :: String
--   , phyloTerms        :: String
--   , phyloGroups       :: String
--   , phyloBranches     :: String
--   , objects           :: Array PhyloObject
--   , edges             :: Array PhyloEdge
--   }

derive instance Generic PhyloObject _
instance Eq PhyloObject where eq = genericEq 

instance showPhyloObject :: Show PhyloObject where
  show (PhyloBranch {nodeType}) = nodeType
  show (PhyloGroup  {nodeType}) = nodeType
  show (PhyloPeriod {nodeType}) = nodeType
  show (Default     {_gvid}) = "DefaultNode"

data PhyloObject = PhyloBranch  
                { _gvid :: Int
                , nodeType :: String
                } | PhyloGroup   
                { _gvid :: Int
                , nodeType :: String
                } | PhyloPeriod  
                { _gvid :: Int
                , nodeType :: String
                } | Default 
                {
                  _gvid :: Int
                }

--instance JSON.ReadForeign PhyloObject where
--  readImpl f = do
--    { _gvid, nodeType: mNodeType } :: { _gvid :: Int, nodeType :: Maybe String } <- JSON.readImpl f
--    case mNodeType of
--      Nothing -> pure $ Default { _gvid }
--      Just nodeType -> do
--        { bId: mBId } :: { bId :: Maybe String } <- JSON.readImpl f
--        case mBId of
--          Nothing -> pure $ Default { _gvid }
--          Just bId ->
--            case nodeType of
--              "branch" -> pure $ PhyloBranch { _gvid, bId, nodeType }
--              "group" -> pure $ PhyloGroup { _gvid, bId, nodeType }
--              "period" -> pure $ PhyloPeriod { _gvid, nodeType }
--              _ -> pure $ Default { _gvid }

instance JSON.ReadForeign PhyloObject where
  readImpl f = do
    { _gvid, nodeType: mNodeType} :: { _gvid :: Int, nodeType :: Maybe String } <- JSON.readImpl f
    case mNodeType of
      Nothing -> pure $ Default { _gvid }
      Just nodeType ->
        case nodeType of
          "branch" -> pure $ PhyloBranch { _gvid, nodeType }
          "group" -> pure $ PhyloGroup { _gvid, nodeType }
          "period" -> pure $ PhyloPeriod { _gvid, nodeType }
          _ -> pure $ Default { _gvid }

derive instance Generic PhyloEdge _
instance Eq PhyloEdge where eq = genericEq 

data PhyloEdge =  Link  
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | AncestorLink   
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | BranchLink  
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | DefaultLink
                {
                  _gvid :: Int
                }

instance JSON.ReadForeign PhyloEdge where
  readImpl f = do
    { _gvid, edgeType: mEdgeType } :: { _gvid :: Int, edgeType :: Maybe String } <- JSON.readImpl f
    case mEdgeType of
      Nothing -> pure $ DefaultLink { _gvid }
      Just edgeType -> do
        { bId: mBId } :: { bId :: Maybe String } <- JSON.readImpl f
        case mBId of
          Nothing -> pure $ DefaultLink { _gvid }
          Just bId ->
            case edgeType of
              "ancestorLink" -> pure $ AncestorLink { _gvid
                                                    , bId
                                                    , edgeType }
              "branchLink" -> pure $ BranchLink { _gvid
                                                , bId
                                                , edgeType }
              "link" -> pure $ Link { _gvid
                                    , bId
                                    , edgeType }
              _ -> pure $ DefaultLink { _gvid }

--newtype PhyloData = PhyloData
--  { name                :: String
--  , phyloDocs           :: Int
--  , phyloFoundations    :: Int
--  , phyloPeriods        :: Int
--  , phyloTerms          :: Int
--  , phyloGroups         :: Int
--  , phyloBranches       :: Int
--  , phyloOBranches      :: Array PhyloObject
--  , phyloOGroups        :: Array PhyloObject
--  , phyloOPeriods       :: Array PhyloObject
--  , phyloOLinks         :: Array PhyloEdge
--  , phyloOAncestorLinks :: Array PhyloEdge
--  , phyloOBrancheLinks  :: Array PhyloEdge
--  }
--
--
--type PhyloConfig =
--  {
--
--  }
-- getBranch :: 
-- type PhyloObjects =
--   { _gvid    :: Int
--   , shape    :: Maybe String
--   , nodeType :: Maybe String
--   }
-- 
-- type PhyloEdges =
--   { _gvid    :: Int
--   , color    :: Maybe String
--   , edgeType :: Maybe String
--   }

-- data PhyloObjects = Branch 
--                 { _gvid :: Int
--                 ,  name :: String
--                 , nodeType :: Maybe String
--                 } | Group
--                 { _gvid :: Int
--                 ,  name :: String
--                 , nodeType :: Maybe String
--                 } | Period
--                 { _gvid :: Int
--                 ,  name :: String
--                 , nodeType :: Maybe String
--                 }
-- 
-- data PhyloEdges = Link
--                 { _gvid    :: Int
--                 ,  head    :: Int
--                 , edgeType :: Maybe String
--                 } | BranchLink
--                 { _gvid    :: Int
--                 ,  head    :: Int
--                 , edgeType :: Maybe String
--                 } | AncestorLink
--                 { _gvid    :: Int
--                 ,  head    :: Int
--                 , edgeType :: Maybe String
--                 }
-- 
-- instance JSON.ReadForeign PhyloEdges where
--   readImpl f = do
--     inst <- JSON.readImpl f
--     pure $ PhyloEdges 

fetchPhyloJSON :: Aff (Maybe PhyloJSON)
fetchPhyloJSON = do
  result <- AX.request (AX.defaultRequest { url = "http://localhost:5501/data/knowledge-phylomemy.json", method = Left GET, responseFormat = ResponseFormat.string })
  case result of
    Left err -> do
      liftEffect $ log $ "Phylo failed to load error : " <> AX.printError err
      pure Nothing
    Right response -> liftEffect $ do
      case JSON.readJSON response.body of
        Left err -> do
          log $ "Fail with error: " <> show err
          pure Nothing
        Right (r :: PhyloJSON) -> do
          log $ "name of phylo is : " <> show r.name
          log $ getBranches r.objects
          log $ getGroups r.objects
          log $ getPeriods r.objects
          pure $ Just r

isBranch :: PhyloObject -> Boolean
isBranch (PhyloBranch _) = true
isBranch _ = false

getBranches :: Array PhyloObject -> Array PhyloObject
getBranches x = filter isBranch x

isGroup :: PhyloObject -> Boolean
isGroup (PhyloGroup _) = true
isGroup _ = false

getGroups :: Array PhyloObject -> Array PhyloObject
getGroups x = filter isGroup x

isPeriod :: PhyloObject -> Boolean
isPeriod (PhyloPeriod _) = true
isPeriod _ = false

getPeriods :: Array PhyloObject -> Array PhyloObject
getPeriods x = filter isPeriod x

-- fetchmyJSON = do
--   result <- AX.request (AX.defaultRequest { url = "http://localhost:5501/data/myJSON.json", method = Left GET, -- responseFormat = ResponseFormat.string })
--   case result of
--     Left err -> do
--       liftEffect $ log $ "data failed to load error : " <> AX.printError err
--       pure Nothing
--     Right response -> liftEffect $ do
--       case JSON.readJSON response.body of
--         Left err -> do
--           log $ "Fail with error: " <> show err
--           pure Nothing
--         Right (r :: myJSON) -> do
--           log $ "name of data is : " <> show r.name
--           pure $ Just r

-- fetchPhylo' =
--   launchAff_
--     $ do
--       result <- AX.get ResponseFormat.string "http://localhost:5501/data/knowledge-phylomemy.json"
--       case result of
--         Left err -> do
--           liftEffect $ log $ "GET /api response failed to decode: " <> AX.printError err
--         Right result -> liftEffect $ do
--           case JSON.readJSON result.body of
--             Right (r :: PhyloJSON) -> do
--               pure $ Just r.name
--             Left e -> do
--               log $ "Couldn't parse Phylo" <> show e

-- getPhylo :: (Maybe PhyloJSON) -> PhyloJSON
-- getPhylo (Just phylo) = phylo
-- getPhylo Nothing = Nothing

logPhyloJSON :: Effect Unit
logPhyloJSON = do
  launchAff_ $ do
    mr <- fetchPhyloJSON
    liftEffect $ do
      case mr of
        Nothing -> log "Nothing"
        Just r  -> log2 "r" r.name


-- getPhyloName = 
--   launchAff_ $ do
--     mr <- fetchPhyloJSON
--       case mr of
--         Nothing -> log "Nothing"
--         pure $ Just r

-- convertJSONtoData = do
--   launchAff_ $ do
--     json <- fetchPhyloJSON
--     liftEffect $ do
--       case json of
--         Nothing -> log "Failed to convert"
--         Just j -> pure $ PhyloData

-- fromJSONtoData = do
--   launchAff_ $ do
--     mr <- fetchPhyloJSON
--       case mr of
--         Nothing -> Nothing
--         Just r -> 

-- getName :: (Maybe PhyloJSON) -> String
-- getName phylo = case phylo of
--                    Nothing -> Nothing
--                    Just phylo -> phylo.name
-- getObjects :: PhyloJSON -> Array PhyloObjects
-- getObjects 
-- 


